from __future__ import print_function

import tempfile
import textwrap
import shutil
import pathlib
import glob

import osc.conf
import osc.core


# TODO: Turn into class and make it context-aware

def configure(obs_server_addr, obs_auth_user, obs_auth_pass,
              insecure_connection=False):
    # Configure the Open Build Service Commander

    # SSL check setting
    obs_ssl_check = "1"
    if insecure_connection is True:
            obs_ssl_check = "0"

    # Open Build Service Commander configuration file data
    obs_server_config = textwrap.dedent("""\
    [general]
    apiurl = {server}
    do_package_tracking = 1


    [{server}]
    user = {user}
    pass = {password}
    keyring = 0
    sslcertck = {ssl_check}
    """).format(server=obs_server_addr,
                user=obs_auth_user,
                password=obs_auth_pass,
                ssl_check=obs_ssl_check)

    # Make a temporary config file to set up for using with osc and use it
    with tempfile.NamedTemporaryFile(suffix=".oscrc") as oscconf:
            oscconf.write(obs_server_config)
            oscconf.flush()
            osc.conf.get_config(override_conffile=oscconf.name)

    return oscconf


def checkout_obs_package(obsconf, obs_server_addr, obs_project, obs_package,
                         obs_auth_user, src_repo_owner, src_repo_name,
                         create_obs_pkg_if_none=False):

    obs_newpkg_config = """
    <package name="{1}" project="{0}">
      <title>{1} for {0}</title>
      <description>{1} for {0} created by {2}. Code is at {3}/{4}</description>
    </package>
    """.format(obs_project, obs_package, obs_auth_user, src_repo_owner,
               src_repo_name)

    # Check to see if the package space exists on OBS
    osc_prj_packagelist = list(osc.core.meta_get_packagelist(
                                            obs_server_addr,
                                            prj=obs_project
                                        ))

    if obs_package in osc_prj_packagelist:
            print("Package is configured remotely, proceeding to checkout...")
    else:
        # If it doesn't exist and we are allowing arbitrary package
        # space creation, create the package space and proceed onward
        if create_obs_pkg_if_none is True:
            print("Package is not configured remotely, constructing package!")
            osc.core.edit_meta(metatype="pkg", data=obs_newpkg_config,
                               edit=False, apiurl=obs_server_addr,
                               path_args=(obs_project, obs_package))
            print("Package space created, continuing to checkout...")
        else:
            print("Package is not configured remotely!")
            return False

    # Create directory to check out package sources
    pkg_checkout_dir = tempfile.mkdtemp(suffix="{0}-{1}"
                                        .format(obs_project,
                                                obs_package))

    # Check out the package sources
    osc.core.checkout_package(apiurl=obs_server_addr,
                              project=obs_project,
                              package=obs_package,
                              outdir=pkg_checkout_dir)

    # Return osc Package object
    return osc.core.Package(workingdir=pkg_checkout_dir)


def update_files_for_obs_package(obs_working_pkg, obs_package_ver, sources,
                                 service_file):
    # Purge all sources and the _service file
    prjfiles = glob.glob("{0}/*".format(obs_working_pkg.absdir))

    for pf in prjfiles:
        obs_working_pkg.delete_file(pathlib.PurePath(pf).name)

    # Send changes to OBS

    # Copy files to OBS package space
    for source in sources:
        shutil.copy2(source, obs_working_pkg.absdir)

    # Copy _service file to OBS package space
    shutil.copy2(service_file, "{0}/_service".format(obs_working_pkg.absdir))

    # Add files
    for source in sources:
        try:
            obs_working_pkg.addfile(pathlib.PurePath(source).name)
        except osc.oscerr.PackageFileConflict:
            print("Already tracked source... continuing!")
            pass

    try:
        obs_working_pkg.addfile("_service")
    except osc.oscerr.PackageFileConflict:
        print("Already tracked service file... continuing!")
        pass

    # Commit changes
    obs_working_pkg.commit(msg="Committed {0}-{1}"
                           .format(obs_working_pkg.name, obs_package_ver))
